# Global macros
%define debug_package %{nil}
%{!?libdir:%define libdir /usr/local/lib}
%{!?dpdk_build_path:%define dpdk_build_path "dpdk/build"}
%{!?dpdk_path:%define dpdk_path "dpdk"}

%define commit0 c0dd7d8a61a257e1eff39ad0bbadf4d75ea13652
%define date 20240925
%define shortcommit0 %(c=%{commit0}; echo ${c:0:7})

%define ver 24.01.1


# Spec metadata
Name:           spdk
Version:        %{ver}
Release:        %{date}git%{shortcommit0}%{?dist}
Summary:        Storage Performance Development Kit

# This is a minimal set of requirements needed for SPDK apps to run when built with
# default configuration. These are also predetermined by rpmbuild. Extra requirements
# can be defined through a comma-separated list passed via $requirements when building
# the spec.
Requires: clang
Requires: glibc
Requires: libaio
Requires: libgcc
Requires: libstdc++
Requires: libuuid
Requires: ncurses-libs
Requires: numactl-libs
Requires: openssl-libs
Requires: zlib

BuildRequires: gcc-c++
BuildRequires: librados2-devel
BuildRequires: librbd1-devel
BuildRequires: libuuid-devel
BuildRequires: openssl-devel
BuildRequires: python3-devel
BuildRequires: python3-pip

BuildRequires: meson
BuildRequires: libaio-devel
BuildRequires: libiscsi-devel
BuildRequires: ncurses-devel
BuildRequires: json-c-devel
BuildRequires: libcmocka-devel

BuildRequires: clang-devel
BuildRequires: python3-pyelftools
BuildRequires: zlib-devel
BuildRequires: numactl-devel
BuildRequires: libarchive-devel


ExclusiveArch:  x86_64 ppc64le


License:       BSD
URL:           https://github.com/ceph/spdk
Source0:       spdk-%{commit0}.tar.bz2
Patch0:        0001-dpdk-disable-x86-rdseed-avx512.patch

%description

The Storage Performance Development Kit (SPDK) provides a set of tools and libraries for
writing high performance, scalable, user-mode storage applications. It achieves high
performance by moving all of the necessary drivers into userspace and operating in a
polled mode instead of relying on interrupts, which avoids kernel context switches and
eliminates interrupt handling overhead.

%prep
%setup -n spdk-%{commit0}
%patch0

%build

cfs() {
	(($# > 1)) || return 0

	local dst=$1 f

	mkdir -p "$dst"
	shift; for f; do [[ -e $f ]] && cp -a "$f" "$dst"; done
}

cl() {
	[[ -e $2 ]] || return 0

	cfs "$1" $(find "$2" -name '*.so*' -type f -o -type l | grep -v .symbols)
}

# Rely mainly on CONFIG

./configure \
%ifarch x86_64
    --target-arch="x86-64-v2" \
%endif
    --with-rbd \
    --with-shared \
    --with-virtio \
    --without-nvme-cuse \
    --disable-tests \
    --disable-unit-tests \
    --disable-examples \
    --enable-debug

make
make DESTDIR=%{buildroot} install

cfs %{buildroot}/usr/local/lib/dpdk %{dpdk_build_path}/lib/*
# Special case for SPDK_RUN_EXTERNAL_DPDK setup
cl %{buildroot}/usr/local/lib/dpdk %{dpdk_path}/intel-ipsec-mb/
cl %{buildroot}/usr/local/lib/dpdk %{dpdk_path}/isa-l/


rm -f %{buildroot}/usr/local/lib/lib*.a

rm -r %{buildroot}/usr/local/share/dpdk/examples

# And some useful setup scripts SPDK uses
mkdir -p %{buildroot}/usr/libexec/spdk
mkdir -p %{buildroot}/etc/bash_completion.d
mkdir -p %{buildroot}/etc/profile.d
mkdir -p %{buildroot}/etc/ld.so.conf.d
mkdir -p %{buildroot}%{python3_sitelib}

cat <<-EOF > %{buildroot}/etc/ld.so.conf.d/spdk.conf
%{libdir}
/usr/local/lib/dpdk
EOF

cat <<-'EOF' > %{buildroot}/etc/profile.d/spdk_path.sh
PATH=$PATH:/usr/libexec/spdk/scripts
PATH=$PATH:/usr/libexec/spdk/scripts/vagrant
PATH=$PATH:/usr/libexec/spdk/test/common/config
export PATH
EOF

cfs %{buildroot}/usr/libexec/spdk scripts
ln -s /usr/libexec/spdk/scripts/bash-completion/spdk %{buildroot}/etc/bash_completion.d/

# We need to take into the account the fact that most of the scripts depend on being
# run directly from the repo. To workaround it, create common root space under dir
# like /usr/libexec/spdk and link all potential relative paths the script may try
# to reference.

# setup.sh uses pci_ids.h
ln -s /usr/local/include %{buildroot}/usr/libexec/spdk

%files
/usr/local/bin/*
/usr/local/lib/python%{python3_version}/site-packages/spdk*/*


%package devel
Summary: SPDK development libraries and headers

%description devel
SPDK development libraries and headers

%files devel
/usr/local/include/*
%{libdir}/pkgconfig/*.pc
%{libdir}/*.so*
/etc/ld.so.conf.d/spdk.conf
%dir %{libdir}/dpdk/pmds-24.0
%{libdir}/dpdk/pmds-24.0/*.so*

%post devel
ldconfig

%package scripts
Summary: SPDK scripts and utilities

%description scripts
SPDK scripts and utilities

%files scripts
/usr/libexec/spdk/*
/etc/profile.d/*
/etc/bash_completion.d/*

%post scripts
ldconfig

%package dpdk-libs
Summary: DPDK libraries

%description dpdk-libs
DPDK libraries

%files dpdk-libs
/usr/local/lib/dpdk

%post dpdk-libs
ldconfig



%changelog
* Tue Sep 17 2024 Justin Caratzas <jcaratza@ibm.com>
- Update SPDK to sha c0dd7d8a61a257e1eff39ad0bbadf4d75ea13652

* Mon Sep 09 2024 Justin Caratzas <jcaratza@ibm.com>
- Update SPDK to sha cd16195a3dc914ea445e90b9cc8ba3211e17b267

* Thu May 30 2024 Justin Caratzas <jcaratza@ibm.com>
- add --enable-debug to configure

* Tue May 28 2024 Justin Caratzas <jcaratza@ibm.com>
- Update SPDK to sha 4888878f6c4e2c7561d0d27333873df25a8eea7a

* Thu May 2 2024 Justin Caratzas <jcaratza@ibm.com>
- Use dpdk in spdk source
- Add dpdk-libs subpackage
- Add --target-arch="x86-64-v2" arg to ./configure

* Sun Apr 21 2024 Justin Caratzas <jcaratza@ibm.com>
- Update SPDK to sha dade5b4c96a647e63ef96d9a5e05f3b64ea0598a

* Wed Mar 20 2024 Justin Caratzas <jcaratza@ibm.com>
- Update SPDK to sha 85a10c98e70a1d3bc6c166e2f2939b3e5bae37dc

* Tue Feb 20 2024 Justin Caratzas <jcaratza@ibm.com>
- Update SPDK to sha 6769b6f51c9508981243042396ec2b2f097a5745

* Thu Sep 28 2023 Justin Caratzas <jcaratza@ibm.com>
- Update SPDK to sha 668268f74ea147f3343b9f8136df3e6fcc61f4cf
- Added package_tarball script

* Wed Sep 13 2023 Justin Caratzas <jcaratza@ibm.com>
- Update SDPK to sha 186986cf1044eaaf73e1c67ebcf3a7ce2f1376bb

* Tue Feb 16 2021 Michal Berger <michalx.berger@intel.com>
- Initial RPM .spec for the SPDK
